<?php
/*
   +-------------------------------------------------------------------------+
   | Copyright (C) 2004-2015 The Cacti Group                                 |
   |                                                                         |
   | This program is free software; you can redistribute it and/or           |
   | modify it under the terms of the GNU General Public License             |
   | as published by the Free Software Foundation; either version 2          |
   | of the License, or (at your option) any later version.                  |
   |                                                                         |
   | This program is snmpagent in the hope that it will be useful,           |
   | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
   | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
   | GNU General Public License for more details.                            |
   +-------------------------------------------------------------------------+
   | Cacti: The Complete RRDTool-based Graphing Solution                     |
   +-------------------------------------------------------------------------+
   | This code is designed, written, and maintained by the Cacti Group. See  |
   | about.php and/or the AUTHORS file for specific developer information.   |
   +-------------------------------------------------------------------------+
   | http://www.cacti.net/                                                   |
   +-------------------------------------------------------------------------+
*/
define("EVENT_SEVERITY_LOW", 1);
define("EVENT_SEVERITY_MEDIUM", 2);
define("EVENT_SEVERITY_HIGH", 3);
define("EVENT_SEVERITY_CRITICAL", 4);

function plugin_snmpagent_install() {

	/* updates based on user actions */
	api_plugin_register_hook('snmpagent', 'config_settings', 			'snmpagent_config_settings', 			'setup.php');
	api_plugin_register_hook('snmpagent', 'config_insert',				'snmpagent_config_insert', 				'setup.php');
	api_plugin_register_hook('snmpagent', 'config_arrays',				'snmpagent_config_arrays', 				'setup.php');
	api_plugin_register_hook('snmpagent', 'draw_navigation_text',		'snmpagent_draw_navigation_text', 		'setup.php');
	api_plugin_register_hook('snmpagent', 'global_settings_update', 	'snmpagent_global_settings_update', 	'setup.php');
	api_plugin_register_hook('snmpagent', 'device_action_bottom', 		'snmpagent_device_action_bottom', 		'setup.php');
	api_plugin_register_hook('snmpagent', 'api_device_new', 			'snmpagent_api_device_new', 			'setup.php');
	api_plugin_register_hook('snmpagent', 'data_source_action_bottom', 	'snmpagent_data_source_action_bottom', 	'setup.php');
	api_plugin_register_hook('snmpagent', 'graphs_action_bottom', 		'snmpagent_graphs_action_bottom', 		'setup.php');
	/* poller related updates */
	api_plugin_register_hook('snmpagent', 'poller_exiting', 			'snmpagent_poller_exiting', 			'setup.php');
	api_plugin_register_hook('snmpagent', 'poller_bottom', 				'snmpagent_poller_bottom', 				'setup.php');
	api_plugin_register_hook('snmpagent', 'cacti_stats_update', 		'snmpagent_cacti_stats_update', 		'setup.php');
	api_plugin_register_hook('snmpagent', 'utilities_list', 			'snmpagent_utilities_list', 			"setup.php");
	api_plugin_register_hook('snmpagent', 'utilities_action', 			'snmpagent_utilities_action', 			"setup.php");

	api_plugin_register_realm('snmpagent', 'managers.php', 'Plugin -> SNMPAgent: Configure SNMP Notification Receivers', 1);

	snmpagent_system_setup();
	snmpagent_cache_install();
}

function plugin_snmpagent_uninstall () {
	/* nothing to do here - everything will be handled by PIA */
	return true;
}

function plugin_snmpagent_check_config () {
	/* Here we will check to ensure everything is configured */
	snmpagent_check_upgrade();
	return true;
}

function plugin_snmpagent_upgrade () {
	/* Here we will upgrade to the newest version */
	snmpagent_check_upgrade();
	return false;
}

function plugin_snmpagent_version () {
	return snmpagent_version();
}

function snmpagent_check_upgrade () {
	global $config;

	$files = array('index.php', 'plugins.php');
	if (isset($_SERVER['PHP_SELF']) && !in_array(basename($_SERVER['PHP_SELF']), $files)) {
		return;
	}

	$current = plugin_snmpagent_version();
	$current = $current['version'];
	$old = db_fetch_row("SELECT * FROM plugin_config WHERE directory='snmpagent'");

	if (sizeof($old) && $current != $old["version"]) {
		/* if the plugin is installed and/or active */
		if ($old["status"] == 1 || $old["status"] == 4) {
			/* re-register the hooks */
			plugin_snmpagent_install();
			/* perform a database upgrade */
			require_once($config['base_path'] . '/plugins/snmpagent/system/upgrade.php');
			snmpagent_system_upgrade($old["version"]);
		}
		/* update the plugin information */
		$info = plugin_snmpagent_version();
		$id = db_fetch_cell("SELECT id FROM plugin_config WHERE directory='snmpagent'");
		db_execute("UPDATE plugin_config
					SET name='" . $info["longname"] . "',
						author='" . $info["author"] . "',
						webpage='" . $info["homepage"] . "',
						version='" . $info["version"] . "'
					WHERE id='$id'");
	}
}

function snmpagent_check_dependencies() {
	global $plugins, $config;
	return true;
}

function snmpagent_system_setup() {
	global $config;
	require_once($config['base_path'] . '/plugins/snmpagent/system/install.php');
	snmpagent_system_install();
}

function snmpagent_version () {
	return array(
		'name'     => 'snmpagent',
		'version'  => '0.2.3',
		'longname' => 'Cacti SNMP Agent',
		'author'   => 'The Cacti Group',
		'homepage' => 'http://www.cacti.net',
		'email'    => '',
		'url'      => ''
	);
}

function snmpagent_config_settings(){
	global $tabs, $settings, $config, $snmp_auth_protocols, $snmp_priv_protocols;

	$temp = array(
		'snmpagent_general_header' => array(
			'friendly_name' => 'SNMPAgent',
			'method' => 'spacer',
			),
		'snmpagent_path_snmptrap' => array(
			'friendly_name' => 'SNMP Trap Binary Path',
			'description' => 'The path to your snmptrap binary.',
			'method' => 'filepath',
			'max_length' => 255
		),
	);

	if (!isset($tabs["misc"])) {
		$tabs["misc"] = "Misc";
	}

	if (isset($settings["misc"])) {
		$settings["misc"] = array_merge($settings["misc"], $temp);
	}else{
		$settings["misc"] = $temp;
	}
}

function snmpagent_config_insert(){
	global $config, $menu;
	include_once($config["base_path"] . "/plugins/snmpagent/include/constants.php");
	include_once($config["base_path"] . "/plugins/snmpagent/lib/mib_cache.php");
}

function snmpagent_utilities_list() {
	global $config, $colors;

	html_header(array("SNMPAgent Utilities"), 2); ?>

	<tr bgcolor="#<?php print $colors["form_alternate2"];?>">
		<td class="textArea">
			<a href='utilities.php?action=view_snmpagent_cache'>View SNMPAgent Cache</a>
		</td>
		<td class="textArea">
			This shows all objects being handled by the SNMPAgent.
		</td>
	</tr>
	<tr bgcolor="#<?php print $colors["form_alternate1"];?>">
		<td class="textArea">
			<a href='utilities.php?action=view_snmpagent_events'>View SNMPAgent Notification Log</a>
		</td>
		<td class="textArea">
			This menu pick allows you to view the latest events SNMPAgent has handled in relation to the registered notification receivers.
		</td>
	</tr>
	<tr bgcolor="#<?php print $colors["form_alternate2"];?>">
		<td class="textArea">
			<a href='utilities.php?action=rebuild_snmpagent_cache'>Rebuild SNMPAgent Cache</a>
		</td>
		<td class="textArea">
			The snmp cache will be cleared and re-generated if you select this option. Note that it takes another poller run to restore the SNMP cache completely.
		</td>
	</tr>
	<?php
}

function snmpagent_utilities_action($action) {
	global $config, $colors;

	$options = array(
		'view_snmpagent_status',
		'view_snmpagent_cache',
		'rebuild_snmpagent_cache',
		'view_snmpagent_events');

	if ( in_array($action,$options) ) {
		/* ================= input validation ================= */
		input_validate_input_number(get_request_var_request("refresh"));
		/* ==================================================== */

		load_current_session_value("refresh", "sess_snmpagent_utilities_refresh", "60");

		$refresh["seconds"] = $_REQUEST["refresh"];
		$refresh["page"] = "utilities.php?action=view_snmpagent_status";
		include_once($config['base_path'] . "/plugins/snmpagent/lib/utilities.php");
		include_once($config['base_path'] . "/include/top_header.php");

		switch($action) {
			case 'view_snmpagent_cache':
				snmpagent_utilities_run_cache();
				break;
			case 'rebuild_snmpagent_cache';
				snmpagent_utilities_run_cache(true);
				break;
			case 'view_snmpagent_events':
				snmpagent_utilities_run_eventlog();
				break;
		}

		include_once($config['base_path'] . "/include/bottom_footer.php");
		exit;
	}
	return $action;
}

function snmpagent_draw_navigation_text ($nav) {
	$nav["utilities.php:view_snmpagent_cache"] = array("title" => "SNMPAgent Cache", "mapping" => "index.php:,utilities.php:", "url" => "utilities.php", "level" => "1");
	$nav["utilities.php:rebuild_snmpagent_cache"] = array("title" => "SNMPAgent Cache", "mapping" => "index.php:,utilities.php:", "url" => "utilities.php", "level" => "1");
	$nav["utilities.php:view_snmpagent_events"] = array("title" => "SNMPAgent Notification Log", "mapping" => "index.php:,utilities.php:", "url" => "utilities.php", "level" => "1");

	$nav["managers.php:"] = array("title" => "SNMP Notification Receivers", "mapping" => "index.php:", "url" => "utilities.php", "level" => "1");
	$nav["managers.php:edit"] = array("title" => "SNMP Notification Receivers (edit)", "mapping" => "index.php:", "url" => "utilities.php", "level" => "1");
	$nav["managers.php:save"] = array("title" => "SNMP Notification Receivers", "mapping" => "index.php:", "url" => "utilities.php", "level" => "1");
	$nav["managers.php:actions"] = array("title" => "SNMP Notification Receivers", "mapping" => "index.php:", "url" => "utilities.php", "level" => "1");
	return $nav;
}

function snmpagent_config_arrays () {
	global $menu;
	/* show additional console entries only if this plugin has been enabled */
	if(api_plugin_is_enabled('snmpagent')) {
		$menu['Configuration']['plugins/snmpagent/lib/managers.php'] = 'SNMP Notification Receivers';
	}
}

function snmpagent_cacti_stats_update($data){
	$mc = new MibCache();
	/* refresh total stats */
	$mc->object('cactiStatsTotalsDevices')->set( snmpagent_read('cactiStatsTotalsDevices') );
	$mc->object('cactiStatsTotalsDataSources')->set( snmpagent_read('cactiStatsTotalsDataSources') );
	$mc->object('cactiStatsTotalsGraphs')->set( snmpagent_read('cactiStatsTotalsGraphs') );

	/* local polling stats  - does not support distributed environments so far. */
	$mc->object('cactiStatsLocalPollerRuntime')->set($data[0]);

	$index = 1;
	$values = array(
		"cactiStatsPollerRunTime" => $data[0],
		"cactiStatsPollerConcurrentProcesses" => $data[2],
		"cactiStatsPollerThreads" => $data[3],
		"cactiStatsPollerHosts" => $data[4],
		"cactiStatsPollerHostsPerProcess" => $data[5],
		"cactiStatsPollerItems" => $data[6],
		"cactiStatsPollerRrrdsProcessed" => $data[7],
		"cactiStatsPollerUtilization" => round($data[0]/read_config_option("poller_interval", true)*100, 10)
	);
	$mc->table('cactiStatsPollerTable')->row($index)->update($values);
	$mc->object('cactiStatsLastUpdate')->set( time() );
}

function snmpagent_global_settings_update(){
	$mc = new MibCache();
	$mc->object('cactiApplVersion')->set( snmpagent_read('cactiApplVersion') );
	$mc->object('cactiApplSnmpVersion')->set( snmpagent_read('cactiApplSnmpVersion') );
	$mc->object('cactiApplRrdtoolVersion')->set( read_config_option("rrdtool_version", true) );
	$mc->object('cactiApplPollerEnabled')->set( (read_config_option("poller_enabled", true) == "on") ? 1 : 2 );
	$mc->object('cactiApplPollerType')->set( read_config_option("poller_type", true) );
	$mc->object('cactiApplPollerInterval')->set( read_config_option("poller_interval", true) );
	$mc->object('cactiApplPollerMaxProcesses')->set( read_config_option("concurrent_processes", true) );
	$mc->object('cactiApplPollerLoadBalance')->set( (read_config_option("process_leveling", true) == "on") ? 1 : 2 );
	$mc->object('cactiApplSpineMaxThreads')->set( read_config_option("max_threads", true) );
	$mc->object('cactiApplSpineScriptServers')->set( read_config_option("php_servers", true) );
	$mc->object('cactiApplSpineScriptTimeout')->set( read_config_option("script_timeout", true) );
	$mc->object('cactiApplSpineMaxOids')->set( read_config_option("max_get_size", true) );
	$mc->object('cactiApplLastUpdate')->set( time() );
}

function snmpagent_api_device_new($device){
	$mc = new MibCache();
	/* add device to cactiApplDeviceTable and cactiStatsDeviceTable*/
	$device_data = db_fetch_row("SELECT * FROM `host` WHERE id = " . $device["id"]);

	$appl_values = array(
		"cactiApplDeviceIndex" => $device_data["id"],
		"cactiApplDeviceDescription" => $device_data["description"],
		"cactiApplDeviceHostname" => $device_data["hostname"],
		"cactiApplDeviceStatus" => $device_data["status"],
		"cactiApplDeviceEventCount" => $device_data["status_event_count"],
		"cactiApplDeviceFailDate" => $device_data["status_fail_date"],
		"cactiApplDeviceRecoveryDate" => $device_data["status_rec_date"],
		"cactiApplDeviceLastError" => $device_data["status_last_error"],
	);

	$stats_values = array(
		"cactiStatsDeviceIndex" => $device_data["id"],
		"cactiStatsDeviceHostname" => $device_data["hostname"],
		"cactiStatsDeviceMinTime" => $device_data["min_time"],
		"cactiStatsDeviceMaxTime" => $device_data["max_time"],
		"cactiStatsDeviceCurTime" => $device_data["cur_time"],
		"cactiStatsDeviceAvgTime" => $device_data["avg_time"],
		"cactiStatsDeviceTotalPolls" => $device_data["total_polls"],
		"cactiStatsDeviceFailedPolls" => $device_data["failed_polls"],
		"cactiStatsDeviceAvailability" => $device_data["availability"]
	);

	$mc->table('cactiApplDeviceTable')->row($device["id"])->replace($appl_values);
	$mc->object('cactiApplLastUpdate')->set( time() );

	$mc->table('cactiStatsDeviceTable')->row($device["id"])->replace($stats_values);
	$mc->object('cactiStatsTotalsDevices')->set( snmpagent_read('cactiStatsTotalsDevices') );
	$mc->object('cactiStatsLastUpdate')->set( time() );
}

function snmpagent_data_source_action_bottom($data){
	$mc = new MibCache();
	$action = $data[0];
	if($action == "1") {
		/* delete data sources */
		$mc->object('cactiStatsTotalsDataSources')->set( snmpagent_read('cactiStatsTotalsDataSources') );
		$mc->object('cactiStatsTotalsGraphs')->set( snmpagent_read('cactiStatsTotalsGraphs') );
		$mc->object('cactiStatsLastUpdate')->set( time() );
	}elseif($action == "4") {
		/* duplicate data sources */
		$mc->object('cactiStatsTotalsDataSources')->set( snmpagent_read('cactiStatsTotalsDataSources') );
		$mc->object('cactiStatsLastUpdate')->set( time() );
	}
}

function snmpagent_graphs_action_bottom($data){
	$mc = new MibCache();
	$action = $data[0];
	if($action == "1") {
		/* delete graphs */
		$mc->object('cactiStatsTotalsDataSources')->set( snmpagent_read('cactiStatsTotalsDataSources') );
		$mc->object('cactiStatsTotalsGraphs')->set( snmpagent_read('cactiStatsTotalsGraphs') );
		$mc->object('cactiStatsLastUpdate')->set( time() );
	}elseif($action == "3") {
		/* duplicate graphs */
		$mc->object('cactiStatsTotalsGraphs')->set( snmpagent_read('cactiStatsTotalsGraphs') );
		$mc->object('cactiStatsLastUpdate')->set( time() );
	}
}

function snmpagent_device_action_bottom($data){
	$mc = new MibCache();
	$action = $data[0];
	$selected_items = $data[1];
	/* selected items are give by the user - do not forget to validate them! */
	if(sizeof($selected_items)>0) {
		switch($action){
			case "1":
				/* delete devices */
				foreach($selected_items as $device_id) {
					if(is_numeric($device_id)) {
						/* remove all device entries */
						$mc->table('cactiApplDeviceTable')->row($device_id)->delete();
						$mc->table('cactiStatsDeviceTable')->row($device_id)->delete();
					}
				}
				/* update total statistics */
				$mc->object('cactiStatsTotalsDevices')->set( snmpagent_read('cactiStatsTotalsDevices') );
				$mc->object('cactiStatsTotalsDataSources')->set( snmpagent_read('cactiStatsTotalsDataSources') );
				$mc->object('cactiStatsTotalsGraphs')->set( snmpagent_read('cactiStatsTotalsGraphs') );
				$mc->object('cactiStatsLastUpdate')->set( time() );
				break;
			case "2":
				/* enable devices */
				foreach($selected_items as $device_id) {
					if(is_numeric($device_id)) {
						$device_status = db_fetch_cell("SELECT status FROM host WHERE id = " . $device_id);
						$mc->table('cactiApplDeviceTable')->row($device_id)->update(array('cactiApplDeviceStatus' => $device_status));
					}
				}
				$mc->object('cactiApplLastUpdate')->set( time() );
				break;
			case "3":
				/* disable devices */
				foreach($selected_items as $device_id) {
					if(is_numeric($device_id)) {
						$device_status = db_fetch_cell("SELECT status FROM host WHERE id = " . $device_id);
						$mc->table('cactiApplDeviceTable')->row($device_id)->update(array('cactiApplDeviceStatus' => 4));
					}
				}
				$mc->object('cactiApplLastUpdate')->set( time() );
				break;
			case "5":
				/* clear device statisitics */
				$values = array(
					'cactiStatsDeviceMinTime' => '9.99999',
					'cactiStatsDeviceMaxTime' => '0',
					'cactiStatsdeviceCurTime' => '0',
					'cactiStatsDeviceAvgTime' => '0',
					'cactiStatsDeviceTotalPolls' => '0',
					'cactiStatsDeviceFailedPolls' => '0',
					'cactiStatsDeviceAvailability' => '100'
				);
				foreach($selected_items as $device_id) {
					if(is_numeric($device_id)) {
						$mc->table('cactiStatsDeviceTable')->row($device_id)->update($values);
					}
				}
				$mc->object('cactiStatsLastUpdate')->set( time() );
				break;

			default:
				/* nothing to do */
			;
		} //switch
	}
}

function snmpagent_poller_exiting($poller_index = 1){
	$mc = new MibCache();
	$poller = $mc->table('cactiApplPollerTable')->row($poller_index)->select();
	$varbinds = array(
		'cactiApplPollerIndex' => $poller_index,
		'cactiApplPollerHostname' => $poller['cactiApplPollerHostname'],
		'cactiApplPollerIpAddress' => $poller['cactiApplPollerIpAddress']
	);
	snmpagent_notification('cactiNotifyPollerRuntimeExceeding', 'CACTI-MIB', $varbinds, EVENT_SEVERITY_HIGH);
}

function snmpagent_poller_bottom() {
	global $config, $plugins_system;

	if (api_plugin_is_enabled('maint')) {
		include_once($config["base_path"] . '/plugins/maint/functions.php');
	}

	$device_in_maintenance = false;
	$mc = new MibCache();
	/* START: update total device stats table */
	/***** deprecated ******/
	$devicestatus_indices = array( 0=> 0, 1=> 1, 2=> 2, 3=> 3, 4=> 4);
	$current_states = db_fetch_assoc("SELECT status, COUNT(*) as cnt FROM `host` GROUP BY status");
	if($current_states && sizeof($current_states)>0) {
		foreach($current_states as $current_state) {
			$index = $devicestatus_indices[$current_state["status"]];
			$values = array(
				"cactiStatsTotalsDeviceStatusIndex" => $current_state["status"],
				"cactiStatsTotalsDeviceStatusCounter" => $current_state["cnt"]
			);
			$mc->table('cactiStatsTotalsDeviceStatusTable')->row($index)->replace($values);
			unset($devicestatus_indices[$current_state["status"]]);
		}
	}
	if(sizeof($devicestatus_indices)>0) {
		foreach($devicestatus_indices as $status => $index) {
			$values = array(
				"cactiStatsTotalsDeviceStatusIndex" => $status,
				"cactiStatsTotalsDeviceStatusCounter" => 0
			);
			$mc->table('cactiStatsTotalsDeviceStatusTable')->row($index)->replace($values);
		}
	}
	/************************/
	$mc->object('cactiStatsTotalsDeviceStatusUnknown')->set( snmpagent_read('cactiStatsTotalsDeviceStatusUnknown') );
	$mc->object('cactiStatsTotalsDeviceStatusDown')->set( snmpagent_read('cactiStatsTotalsDeviceStatusDown') );
	$mc->object('cactiStatsTotalsDeviceStatusRecovering')->set( snmpagent_read('cactiStatsTotalsDeviceStatusRecovering') );
	$mc->object('cactiStatsTotalsDeviceStatusUp')->set( snmpagent_read('cactiStatsTotalsDeviceStatusUp') );
	$mc->object('cactiStatsTotalsDeviceStatusDisabled')->set( snmpagent_read('cactiStatsTotalsDeviceStatusDisabled') );
	/* END: update total device stats table */

	/* update state and statistics of all devices */
	$mc_dstatus = array();
	$mc_devices = $mc->table('cactiApplDeviceTable')->select(array('cactiApplDeviceIndex', 'cactiApplDeviceStatus'));
	if($mc_devices && sizeof($mc_devices)>0) {
		foreach($mc_devices as $mc_device) {
			$mc_dstatus[$mc_device['cactiApplDeviceIndex']] = $mc_device['cactiApplDeviceStatus'];
		}
	}
	$mc_dfailed = array();
	$mc_device_stats = $mc->table('cactiStatsDeviceTable')->select(array('cactiStatsDeviceIndex','cactiStatsDeviceFailedPolls'));
	if($mc_device_stats && sizeof($mc_device_stats)>0) {
		foreach($mc_device_stats as $mc_device_stat) {
			$mc_dfailed[$mc_device_stat['cactiStatsDeviceIndex']] = $mc_device_stat['cactiStatsDeviceFailedPolls'];
		}
	}

	$devices = db_fetch_assoc("SELECT id, description, hostname, status, disabled, status_event_count, status_fail_date, status_rec_date, status_last_error, min_time, max_time, cur_time, avg_time, total_polls, failed_polls, availability FROM host ORDER BY id ASC");
	if($devices && sizeof($devices)>0) {
		foreach($devices as $device) {
			if(function_exists('plugin_maint_check_cacti_host')) {
				$device_in_maintenance = plugin_maint_check_cacti_host($index);
			}
			if(!$device_in_maintenance) {
				$varbinds = array(
					'cactiApplDeviceIndex' => $device["id"],
					'cactiApplDeviceDescription' => $device["description"],
					'cactiApplDeviceHostname' => $device["hostname"],
					'cactiApplDeviceLastError' => $device["status_last_error"]
				);
				if($device["failed_polls"] > $mc_dfailed[$device["id"]]) {
					snmpagent_notification('cactiNotifyDeviceFailedPoll', 'CACTI-MIB', $varbinds);
				}
				if($mc_dstatus[$device["id"]] == HOST_UP && $device["status"] == HOST_DOWN ) {
					snmpagent_notification('cactiNotifyDeviceDown', 'CACTI-MIB', $varbinds, EVENT_SEVERITY_HIGH);
				}elseif ($mc_dstatus[$device["id"]] == HOST_DOWN && $device["status"] == HOST_RECOVERING ){
					snmpagent_notification('cactiNotifyDeviceRecovering', 'CACTI-MIB', $varbinds);
				}
			}

			$values = array(
				"cactiApplDeviceStatus" => ($device["disabled"] == 'on') ? 4 : $device["status"],
				"cactiApplDeviceEventCount" => $device["status_event_count"],
				"cactiApplDeviceFailDate" => $device["status_fail_date"],
				"cactiApplDeviceRecoveryDate" => $device["status_rec_date"],
				"cactiApplDeviceLastError" => $device["status_last_error"]
			);
			$mc->table('cactiApplDeviceTable')->row($device["id"])->update($values);
			$values = array(
				"cactiStatsDeviceMinTime" => $device["min_time"],
				"cactiStatsDeviceMaxTime" => $device["max_time"],
				"cactiStatsDeviceCurTime" => $device["cur_time"],
				"cactiStatsDeviceAvgTime" => $device["avg_time"],
				"cactiStatsDeviceTotalPolls" => $device["total_polls"],
				"cactiStatsDeviceFailedPolls" => $device["failed_polls"],
				"cactiStatsDeviceAvailability" => $device["availability"]
			);
			$mc->table('cactiStatsDeviceTable')->row($device["id"])->update($values);
		}
	}

	/* get a list of all plugins available on that system */
	$pluginslist = snmpagent_get_pluginslist();
	/* truncate plugin mib table */
	$mc->table('cactiApplPluginTable')->truncate();
	/* refill plugin mib table */
	if($pluginslist && sizeof($pluginslist)>0) {
		$i = 1;
		foreach($pluginslist as $plugin) {
			$values = array(
				"cactiApplPluginIndex" => $i,
				"cactiApplPluginType" => (in_array($plugin["directory"], $plugins_system) ? 1 : 2),
				"cactiApplPluginName" => $plugin["directory"],
				"cactiApplPluginStatus" => $plugin["status"],
				"cactiApplPluginVersion" => $plugin["version"]
			);
			$mc->table('cactiApplPluginTable')->row($i)->insert($values);
			$i++;
		}
	}
	$mc->object('cactiApplLastUpdate')->set( time() );

	$recache_stats = db_fetch_cell("SELECT value FROM settings WHERE name = 'stats_recache'");
	if($recache_stats) {
		list($time, $hosts) = explode(" ", $recache_stats);
		$time = str_replace("RecacheTime:", "", $time);
		$hosts = str_replace("HostsRecached:", "", $hosts);
	}
	$mc->object('cactiStatsRecacheTime')->set($time);
	$mc->object('cactiStatsRecachedHosts')->set($hosts);
	$mc->object('cactiStatsLastUpdate')->set( time() );

	/* clean up the notification log */
	$snmp_notification_managers = db_fetch_assoc("SELECT id, max_log_size FROM plugin_snmpagent_managers");
	if($snmp_notification_managers && sizeof($snmp_notification_managers)>0) {
		foreach($snmp_notification_managers as $snmp_notification_manager) {
			db_execute("DELETE FROM plugin_snmpagent_notifications_log WHERE manager_id = " . $snmp_notification_manager["id"] . " AND `time` <= " . (time()-86400*$snmp_notification_manager["max_log_size"]) );
		}
	}
}

function snmpagent_get_pluginslist(){
	global $config, $plugins, $plugins_system;
	/* update the list of known plugins only once per polling cycle. In all other cases we would
	   have to create too many new hooks to update that MIB table just in time.
	   We have to do the same like function plugins_load_temp_table(), which will not be available
	   during the execution of that function. */

	$pluginslist = array();
	$registered_plugins = db_fetch_assoc('SELECT * FROM plugin_config ORDER BY name');
	foreach ($registered_plugins as $t) {
		$pluginslist[$t['directory']] = $t;
	}

	$path = $config['base_path'] . '/plugins/';
	$dh = opendir($path);
	if ($dh !== false) {
		while (($file = readdir($dh)) !== false) {
			if ((is_dir("$path/$file")) && (file_exists("$path/$file/setup.php")) && (!array_key_exists($file, $pluginslist))) {
				include_once("$path/$file/setup.php");
				if (!function_exists('plugin_' . $file . '_install') && function_exists($file . '_version')) {
					$function = $file . '_version';
					$cinfo = $function();
					if (!isset($cinfo['author']))   $cinfo['author']   = 'Unknown';
					if (!isset($cinfo['homepage'])) $cinfo['homepage'] = 'Not Stated';
					if (isset($cinfo['webpage']))   $cinfo['homepage'] = $cinfo['webpage'];
					if (!isset($cinfo['longname'])) $cinfo['longname'] = ucfirst($file);
					$cinfo['status'] = -2; /* old PIA -- disabled */
					if (in_array($file, $plugins)) {
						$cinfo['status'] = -1; /* old PIA -- enabled */
					}
					$cinfo['directory'] = $file;
					$pluginslist[$file] = $cinfo;

				} elseif (function_exists('plugin_' . $file . '_install') && function_exists('plugin_' . $file . '_version')) {
					$function = $file . '_version';
					$cinfo = $function();
					$cinfo['status'] = 0;
					if (!isset($cinfo['author']))   $cinfo['author']   = 'Unknown';
					if (!isset($cinfo['homepage'])) $cinfo['homepage'] = 'Not Stated';
					if (isset($cinfo['webpage']))   $cinfo['homepage'] = $cinfo['webpage'];
					if (!isset($cinfo['longname'])) $cinfo['homepage'] = ucfirst($file);
					$cinfo['directory'] = $file;
					$pluginslist[$file] = $cinfo;
				}
			}
		}
		closedir($dh);
	}
	return $pluginslist;
}

/**
 * snmpagent_cache_setup()
 * Generates a SNMP caching tables reflecting all objects of the Cacti MIB
 * @return
 */
function snmpagent_cache_install(){
	global $config, $plugins_system;
	include_once($config["base_path"] . "/plugins/snmpagent/lib/mib_cache.php");

	/* drop everything */
	db_execute("TRUNCATE `plugin_snmpagent_cache`");
	db_execute("TRUNCATE `plugin_snmpagent_mibs`;");
	db_execute("TRUNCATE `plugin_snmpagent_cache_notifications`;");
	db_execute("TRUNCATE `plugin_snmpagent_cache_textual_conventions`;");

	$mc = new MibCache();
	$mc->install($config["base_path"] . '/plugins/snmpagent/mibs/CACTI-MIB');
	$mc->install($config["base_path"] . '/plugins/snmpagent/mibs/CACTI-SNMPAGENT-MIB');
	snmpagent_cache_init();

	/* call install routine of plugins supporting the SNMPagent */
	do_hook('snmpagent_cache_install');
}

function snmpagent_cache_rebuilt(){
	snmpagent_cache_install();
}

function snmpagent_cache_init(){
	/* fill up the cache with a minimum of data data and ignore all values that
	   *  will be updated automatically at the bottom of the next poller run
	*/
	$mc = new MibCache();
	/* update global settings */
	snmpagent_global_settings_update();

	/* add pollers of a distributed system (future) */
	$pollers = db_fetch_assoc("SELECT id FROM poller ORDER BY id ASC");
	if($pollers && sizeof($pollers)>0) {
		foreach($pollers as $poller){
			$poller_data = db_fetch_row("SELECT * FROM poller WHERE id = " . $poller["id"]);
		}
	}else {
		/* this is NOT a distributed system, but it should have at least one local poller. */
		$poller_lastrun = read_config_option("poller_lastrun", true);
		$values = array(
			"cactiApplPollerIndex" => 1,
			"cactiApplPollerHostname" => "localhost",
			"cactiApplPollerIpAddress" => "127.0.0.1",
			"cactiApplPollerLastUpdate" => $poller_lastrun
		);
		$mc->table('cactiApplPollerTable')->row(1)->insert($values);

		$values = array(
			"cactiStatsPollerIndex" => 1,
			"cactiStatsPollerHostname" => "localhost",
			"cactiStatsPollerMethod" => read_config_option("poller_type", true)
		);
		$mc->table('cactiStatsPollerTable')->row(1)->insert($values);
	}

	/* add all devices as devicetable entries to the snmp cache */
	$devices = db_fetch_assoc("SELECT id, description, hostname, disabled, status_event_count, status_fail_date, status_rec_date, status_last_error, min_time, max_time, cur_time, avg_time, total_polls, failed_polls, availability FROM host ORDER BY id ASC");
	if($devices && sizeof($devices)>0) {
		foreach($devices as $device) {
			$device = db_fetch_row("SELECT * FROM host WHERE id = " . $device["id"]);
			/* add device to cactiApplDeviceTable */
			$values = array(
				"cactiApplDeviceIndex" => $device["id"],
				"cactiApplDeviceDescription" => $device["description"],
				"cactiApplDeviceHostname" => $device["hostname"],
				"cactiApplDeviceStatus" => ($device["disabled"] == 'on') ? 4 : $device["status"],
				"cactiApplDeviceEventCount" => $device["status_event_count"],
				"cactiApplDeviceFailDate" => $device["status_fail_date"],
				"cactiApplDeviceRecoveryDate" => $device["status_rec_date"],
				"cactiApplDeviceLastError" => $device["status_last_error"],
			);
			$mc->table('cactiApplDeviceTable')->row($device["id"])->insert($values);

			/* add device to cactiStatsDeviceTable */
			$values = array(
				"cactiStatsDeviceIndex" => $device["id"],
				"cactiStatsDeviceHostname" => $device["hostname"],
				"cactiStatsDeviceMinTime" => $device["min_time"],
				"cactiStatsDeviceMaxTime" => $device["max_time"],
				"cactiStatsDeviceCurTime" => $device["cur_time"],
				"cactiStatsDeviceAvgTime" => $device["avg_time"],
				"cactiStatsDeviceTotalPolls" => $device["total_polls"],
				"cactiStatsDeviceFailedPolls" => $device["failed_polls"],
				"cactiStatsDeviceAvailability" => $device["availability"]
			);
			$mc->table('cactiStatsDeviceTable')->row($device["id"])->insert($values);
		}
	}
}

function snmpagent_read($object){
	switch($object) {
		case "cactiApplVersion":
			$value = db_fetch_cell("SELECT `cacti` FROM `version`");
			break;
		case "cactiApplSnmpVersion":
			$versions = array("net-snmp" => 1, "ucd-snmp" => 2);
			$snmp_version = read_config_option("snmp_version", true);
			$value = $versions[$snmp_version];
			if(function_exists("snmpget")) {
				$value = 3;
			}
			break;
		case "cactiStatsTotalsDevices":
			$value = db_fetch_cell("SELECT COUNT(*) FROM host");
			break;
		case "cactiStatsTotalsDataSources":
			$value = db_fetch_cell("SELECT COUNT(*) FROM data_local");
			break;
		case "cactiStatsTotalsGraphs":
			$value = db_fetch_cell("SELECT COUNT(*) FROM graph_local");
			break;
		case "cactiStatsTotalsDeviceStatusUnknown":
			$value = db_fetch_cell("SELECT COUNT(*) FROM host WHERE status = 0");
			break;
		case "cactiStatsTotalsDeviceStatusDown":
			$value = db_fetch_cell("SELECT COUNT(*) FROM host WHERE status = 1");
			break;
		case "cactiStatsTotalsDeviceStatusRecovering":
			$value = db_fetch_cell("SELECT COUNT(*) FROM host WHERE status = 2");
			break;
		case "cactiStatsTotalsDeviceStatusUp":
			$value = db_fetch_cell("SELECT COUNT(*) FROM host WHERE status = 3");
			break;
		case "cactiStatsTotalsDeviceStatusDisabled":
			$value = db_fetch_cell("SELECT COUNT(*) FROM host WHERE status = 4");
			break;
		default:
			$value = false;
	}
	return $value;
}

function snmpagent_notification( $notification, $mib, $varbinds, $severity = EVENT_SEVERITY_MEDIUM){
	global $config;

	if(isset($config["snmpagent"]["notifications"]["ignore"][$notification])) {
		return false;
	}

	$path_snmptrap = read_config_option("snmpagent_path_snmptrap");

	if( !in_array( $severity, array( EVENT_SEVERITY_LOW, EVENT_SEVERITY_MEDIUM, EVENT_SEVERITY_HIGH, EVENT_SEVERITY_CRITICAL ) ) ) {
		if(read_config_option('log_verbosity')>POLLER_VERBOSITY_NONE) {
			cacti_log('ERROR: Unknown event severity: "' . $severity . '" for ' . $notification . ' (' . $mib . ')', false, 'SNMPAGENT');
		}
		return false;
	}

	$enterprise_oid = db_fetch_cell("SELECT oid from plugin_snmpagent_cache where `name` = '" . $notification . "' AND `mib` = '" . $mib ."'");
	if(!$enterprise_oid) {
		/* system does not know this event */
		if(read_config_option('log_verbosity')>POLLER_VERBOSITY_NONE) {
			cacti_log('ERROR: Unknown event: ' . $notification . ' (' . $mib . ')', false, 'SNMPAGENT');
		}
		return false;
	}else {
		$branches = explode(".", $enterprise_oid);
		$specific_trap_number = array_pop($branches);
	}

	/* generate a list of SNMP notification receivers listening for this notification */
	$sql = "SELECT plugin_snmpagent_managers.*
				FROM plugin_snmpagent_managers_notifications
					INNER JOIN plugin_snmpagent_managers
					ON (
						plugin_snmpagent_managers.id = plugin_snmpagent_managers_notifications.manager_id
					)
				WHERE plugin_snmpagent_managers.disabled = 0 AND plugin_snmpagent_managers_notifications.notification = '$notification' AND plugin_snmpagent_managers_notifications.mib = '$mib'";

	$notification_managers = db_fetch_assoc($sql);

	if(!$notification_managers) {
		/* To bad! Nobody wants to hear our message. :( */
		if(in_array($severity, array(EVENT_SEVERITY_HIGH, EVENT_SEVERITY_CRITICAL))) {
			if(read_config_option('log_verbosity')>POLLER_VERBOSITY_NONE) {
				cacti_log('WARNING: No notification receivers configured for event: ' . $notification . ' (' . $mib . ')', false, 'SNMPAGENT');
			}
		}else {
			/* keep notifications of a lower/medium severity in mind to make a quicker decision next time */
			$config["snmpagent"]["notifications"]["ignore"][$notification] = 1;
		}
		return false;
	}

	$registered_var_binds = array();
	/* get a list of registered var binds */
	$sql = "SELECT
					plugin_snmpagent_cache_notifications.attribute,
					plugin_snmpagent_cache.oid,
					plugin_snmpagent_cache.type,
					plugin_snmpagent_cache_textual_conventions.type as tcType
			FROM plugin_snmpagent_cache_notifications
				LEFT JOIN plugin_snmpagent_cache
				ON (
					plugin_snmpagent_cache.mib = plugin_snmpagent_cache_notifications.mib
					AND
					plugin_snmpagent_cache.name = plugin_snmpagent_cache_notifications.attribute
				)
				LEFT JOIN plugin_snmpagent_cache_textual_conventions
				ON (
					plugin_snmpagent_cache.mib = plugin_snmpagent_cache_textual_conventions.mib
					AND
					plugin_snmpagent_cache.type = plugin_snmpagent_cache_textual_conventions.name
				)
			WHERE plugin_snmpagent_cache_notifications.name = '$notification' AND plugin_snmpagent_cache_notifications.mib = '$mib'
			ORDER BY plugin_snmpagent_cache_notifications.sequence_id";

	$reg_var_binds = db_fetch_assoc($sql);

	if($reg_var_binds && sizeof($reg_var_binds)>0) {
		foreach($reg_var_binds as $reg_var_bind) {
			$registered_var_binds[$reg_var_bind["attribute"]] = array(
				"oid" => $reg_var_bind["oid"],
				"type" => ($reg_var_bind["tcType"]) ? $reg_var_bind["tcType"] : $reg_var_bind["type"]
			);
		}
	}

	$difference = array_diff( array_keys($registered_var_binds), array_keys($varbinds));

	if(sizeof($difference) == 0) {
		/* order the managers by message type to send out all notifications immmediately. Informs
		   will take more processing time.
		*/
		$sql = "SELECT plugin_snmpagent_managers.* FROM plugin_snmpagent_managers_notifications
					INNER JOIN plugin_snmpagent_managers
					ON (
						plugin_snmpagent_managers.id = plugin_snmpagent_managers_notifications.manager_id
					)
				WHERE plugin_snmpagent_managers_notifications.notification = '$notification' AND plugin_snmpagent_managers_notifications.mib = '$mib'
				ORDER BY plugin_snmpagent_managers.snmp_message_type";

		$notification_managers = db_fetch_assoc($sql);

		if($notification_managers && sizeof($notification_managers)>0) {

			include_once($config["library_path"] . "/poller.php");

			/*
			TYPE: one of i, u, t, a, o, s, x, d, b
				i: INTEGER, u: unsigned INTEGER, t: TIMETICKS, a: IPADDRESS
				o: OBJID, s: STRING, x: HEX STRING, d: DECIMAL STRING, b: BITS
				U: unsigned int64, I: signed int64, F: float, D: double
			*/
			$smi2netsnmp_datatypes = array(
				"integer" 			=> "i",
				"integer32"			=> "i",
				"unsigned32" 		=> "u",
				"gauge" 			=> "i",
				"gauge32" 			=> "i",
				"counter" 			=> "i",
				"counter32" 		=> "i",
				"counter64" 		=> "I",
				"timeticks" 		=> "t",
				"octect string" 	=> "s",
				"opaque"			=> "s",
				"object identifier" => "o",
				"ipaddress" 		=> "a",
				"networkaddress" 	=> "IpAddress",
				"bits" 				=> "b",
				"displaystring" 	=> "s",
				"physaddress" 		=> "s",
				"macaddress" 		=> "s",
				"truthvalue" 		=> "i",
				"testandincr" 		=> "i",
				"autonomoustype" 	=> "o",
				"variablepointer" 	=> "o",
				"rowpointer" 		=> "o",
				"rowstatus" 		=> "i",
				"timestamp" 		=> "t",
				"timeinterval" 		=> "i",
				"dateandtime" 		=> "s",
				"storagetype" 		=> "i",
				"tdomain" 			=> "o",
				"taddress" 			=> "s"
			);

			$log_notification_varbinds = "";
			$snmp_notification_varbinds = "";

			foreach($notification_managers as $notification_manager) {
				if(!$snmp_notification_varbinds) {
					foreach($registered_var_binds as $name => $attributes ) {
						$snmp_notification_varbinds .= " " . $attributes["oid"] . " " . $smi2netsnmp_datatypes[strtolower($attributes["type"])] . " \"" . str_replace('"', "'", $varbinds[$name]) . "\"";
						$log_notification_varbinds .= $name . ":\"" . str_replace('"', "'", $varbinds[$name]) . "\" ";
					}
				}

				if($notification_manager["snmp_version"] == 1 ) {
					$args = " -v 1 -c " . $notification_manager["snmp_community"] . " " . $notification_manager["hostname"] . ":" . $notification_manager["snmp_port"] . " " . $enterprise_oid . " \"\" 6 " . $specific_trap_number . " \"\"" . $snmp_notification_varbinds;
				}else if($notification_manager["snmp_version"] == 2 ) {
					$args = " -v 2c -c " . $notification_manager["snmp_community"] . ( ($notification_manager["snmp_message_type"] == 2 )? " -Ci " : "" )  . " " . $notification_manager["hostname"] . ":" . $notification_manager["snmp_port"] . " \"\" " . $enterprise_oid . $snmp_notification_varbinds;
				}else if($notification_manager["snmp_version"] == 3 ) {
					$args = " -v 3 -e " . $notification_manager["snmp_engine_id"] . (($notification_manager["snmp_message_type"] == 2 )? " -Ci " : "" ) .  " -u " . $notification_manager["snmp_username"];

					if( $notification_manager["snmp_auth_password"] && $notification_manager["snmp_priv_password"]) {
						$snmp_security_level = "authPriv";
					}elseif ( $notification_manager["snmp_auth_password"] && !$notification_manager["snmp_priv_password"]) {
						$snmp_security_level = "authNoPriv";
					}else {
						$snmp_security_level = "noAuthNoPriv";
					}
					$args .= " -l " . $snmp_security_level . (($snmp_security_level != "noAuthNoPriv") ? " -a " . $notification_manager["snmp_auth_protocol"] . " -A " . $notification_manager["snmp_auth_password"] : "" ) . (($snmp_security_level == "authPriv")? " -x " . $notification_manager["snmp_priv_protocol"] . " -X " . $notification_manager["snmp_priv_password"] : "")  . " " . $notification_manager["hostname"] . ":" . $notification_manager["snmp_port"] . " \"\" " . $enterprise_oid . $snmp_notification_varbinds;
				}

				/* execute net-snmp to generate this notification in the background */
				exec_background( escapeshellcmd($path_snmptrap), escapeshellcmd($args));

				/* insert a new entry into the notification log for that SNMP receiver */
				$save = array();
				$save["id"]				= 0;
				$save["time"]			= time();
				$save["severity"]		= $severity;
				$save["manager_id"]		= $notification_manager["id"];
				$save["notification"]	= $notification;
				$save["mib"]			= $mib;
				$save["varbinds"]		= mysql_real_escape_string( substr($log_notification_varbinds, 0, 5000) );
				sql_save( $save, 'plugin_snmpagent_notifications_log');

				/* log the net-snmp command for Cacti admins if they wish for */
				if(read_config_option('log_verbosity')>POLLER_VERBOSITY_MEDIUM) {
					cacti_log("NOTE: $path_snmptrap " . str_replace(array($notification_manager["snmp_auth_password"], $notification_manager["snmp_priv_password"]), '********', $args), false, 'SNMPAGENT');
				}
			}
		}
	}else {
		/* mismatching number of var binds */
		if(read_config_option('log_verbosity')>POLLER_VERBOSITY_NONE) {
			cacti_log('ERROR: Incomplete number of varbinds given for event: ' . $notification . ' (' . $mib . ')', false, 'SNMPAGENT');
		}
		return false;
	}
}